/* message.c
 * 
 * version 0.3.2
 *
 * last changed : 11/05/2001
 *
 * currently displays the text you give as the first argument on the
 * cmdline
 * if you give a second argument, it shows that pixmap
 * current addition : try to get an icon there; currently fixed to image.xpm
 */

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* default values */

#define IMG_X 0
#define IMG_Y 0
#define IMG_W 50
#define IMG_H 50

#define MSG_X 50
#define MSG_Y 0
#define MSG_W 950
#define MSG_H 50

XFontStruct* font_info;       /* Font structure, used for drawing text.    */

/* create two windows, one for the icon and one for the message
 * put them next to each other
 * display the icon in the first
 * display the message graphically in the second
 */


Window 
CreateWindow (Display* dpy, int screen_num, 
              int x, int y, int width, int height)
{
    int vmask;
    XSetWindowAttributes attribs;
    Window w;

    /* message window */
    vmask = CWSaveUnder|CWOverrideRedirect|CWColormap|CWBorderPixel;
    attribs.save_under = True;
    attribs.override_redirect = True;
    attribs.colormap = DefaultColormap (dpy, screen_num);
    attribs.background_pixel = 1000;
    attribs.border_pixel = 0; /* do not care */

    w = XCreateWindow (dpy, DefaultRootWindow (dpy), x, y, width, height,
                       0, CopyFromParent, CopyFromParent, CopyFromParent, 
                       vmask, &attribs);

    XMapWindow (dpy, w);
    return w;
}

GC
create_gc (Display* display, Window win, char color[])
{
  GC gc;                                /* handle of newly created GC.  */
  unsigned long valuemask = 0;          /* which values in 'values'  to */
                                        /* check when creating the GC.  */
                                        /* check when creating the GC.  */
  XGCValues values;                     /* initial values for the GC.   */
  unsigned int line_width = 2;          /* line width for the GC.       */
  int line_style = LineSolid;           /* style for lines drawing and  */
  int cap_style = CapButt;              /* style of the line's edje and */
  int join_style = JoinBevel;           /*  joined lines.               */
  int screen_num = DefaultScreen(display);

  Colormap colormap;
  XColor col;
  
  gc = XCreateGC(display, win, valuemask, &values);
  if (gc < 0) {
        fprintf(stderr, "XCreateGC: \n");
  }
  XSetSubwindowMode ( display, gc, IncludeInferiors );
  /* allocate foreground and background colors for this GC. */
  colormap = DefaultColormap (display, 0);
  XParseColor (display, colormap, color, &col);
  XAllocColor (display, colormap, &col);

//  if (reverse_video) {
    XSetForeground(display, gc, col.pixel);
    XSetBackground(display, gc, BlackPixel(display, screen_num));
/*
  }
  else {
    XSetForeground(display, gc, BlackPixel(display, screen_num));
    XSetBackground(display, gc, WhitePixel(display, screen_num));
  }
*/
  
  /* define the style of lines that will be drawn using this GC. */
  XSetLineAttributes(display, gc,
                     line_width, line_style, cap_style, join_style);
  
  /* define the fill style for the GC. to be 'solid filling'. */
  XSetFillStyle(display, gc, FillSolid);

  return gc;
}

int 
WriteText (Display* display, Window window, XFontStruct* font_info, 
           char *text, GC gc_fg, GC gc_bo)
{
  /* display the text using a shadowy effect */

  Pixmap shapemask;
  GC gc_shape;
  XGCValues gcv;

  gcv.graphics_exposures = False;
  gcv.function = GXcopy;
  gcv.foreground = WhitePixel (display, 0);
  gcv.subwindow_mode = IncludeInferiors;

  shapemask = XCreatePixmap (display, window, 500, 500, 1);
  gc_shape = XCreateGC (display, shapemask, 0 , &gcv);
  XSetFont(display, gc_shape, font_info->fid);
  XSetForeground (display, gc_shape, 0);
  XFillRectangle (display, shapemask, gc_shape, 0, 0, 500, 500);
  XSetForeground (display, gc_shape, 1);

  
  {
    /* variables used for drawing the text strings. */
    int x, y;
    int x_off, y_off;

    //int string_width;
    int font_height;
  
    /* find the height of the characters drawn using this font.        */
    font_height = font_info->ascent + font_info->descent;

    x = 0;
    y = font_height;    

    /* draw black border part */
    for (x_off = 0; x_off <= 3; ++x_off)
    {
      for (y_off = 0; y_off <=3; ++y_off)
      {
        XDrawString (display, window, gc_bo, x + x_off, y + y_off,
                     text,strlen(text));
        XDrawString (display, shapemask, gc_shape, x + x_off, y + y_off,
		     text, strlen (text));
      }
    }

    /* draw white top part */
    for (x_off = 1; x_off <= 2; ++x_off)
    {
      for (y_off = 1; y_off <= 2; ++y_off)
      {
        XDrawString (display, window, gc_fg, x + x_off, y + y_off,
                     text,strlen(text));
      }
    }
    /* flush all pending requests to the X server. */

    XShapeCombineMask (display, window, ShapeBounding, 0, 0, 
                       shapemask, ShapeSet);
    XFlush(display);      
    return 0;
  }


}

int PutXPM 
(Display* display, Window window, GC gc, char* filename)
{
  Pixmap pixmap, shapemask;
  XpmAttributes attributes;
  XGCValues gcv;
  GC gc_clip;

  gcv.graphics_exposures = False;
  gcv.function = GXcopy;
  gcv.foreground = WhitePixel (display, 0);
  gcv.subwindow_mode = IncludeInferiors;

  gc_clip = XCreateGC(display, window, GCFunction
                               |GCGraphicsExposures, &gcv);


  XpmReadFileToPixmap (display, window, filename, &pixmap, &shapemask,
		&attributes);

//  XFillRectangle (display, shapemask, gc_clip, 0, 0, 50, 50);


/*
  XShapeCombineMask (display, window, ShapeClip, 0, 0, shapemask,
ShapeSet);
*/
/*
  XSetClipMask (display, gc_clip, shapemask);
  XSetClipOrigin (display, gc_clip, 0, 0);
  XCopyArea (display, pixmap, window, gc_clip, 0, 0, 100, 100, 0, 0);
*/

  XShapeCombineMask (display, window, ShapeBounding, 0, 0, shapemask,
ShapeSet);
  XCopyArea (display, pixmap, window, gc_clip, 0, 0, 100, 100, 0, 0);
//  XSetWindowBackgroundPixmap (display, window, pixmap);
  XFlush(display);      
  return 1;
}

int main (int argc, char *argv[])
{
  Display* display;             /* pointer to X Display structure.           */
  Window img_w, msg_w;

  char *display_name = getenv("DISPLAY");  /* address of the X display.      */

  int screen_num;               /* number of screen to place the window on.  */
  unsigned int display_width,
               display_height;  /* height and width of the X display.        */

  GC gc, gc_bo;                        /* GC (graphics context) used for
drawing */
                                /*  in our window.                           */
  char* font_name = "-*-helvetica-medium-r-normal-*-24-*-*-*-*-*-*-*"; /*
font to use for drawing text.   */

  if (argc == 1)
  {
    printf ("Please supply a line of text to display !\n");
    return 1;
  }

  /* open connection with the X server. */
  display = XOpenDisplay(display_name);
  if (display == NULL) {
    fprintf(stderr, "%s: cannot connect to X server '%s'\n",
            argv[0], display_name);
    exit(1);
  }
  
  /* get the geometry of the default screen for our display. */
  screen_num = DefaultScreen(display);
  display_width = DisplayWidth(display, screen_num);
  display_height = DisplayHeight(display, screen_num);

  img_w = CreateWindow (display, screen_num, 0, 0, 50, 50);
  msg_w = CreateWindow (display, screen_num, 50, 0, 500, 50);

  /* allocate a new GC (graphics context) for drawing in the window. */

  gc = create_gc(display, msg_w, "yellow");
  gc_bo = create_gc(display, msg_w, "black");
  XSync(display, False);

  /* try to load the given font. */
  font_info = XLoadQueryFont(display, font_name);
  if (!font_info) {
      fprintf(stderr, "XLoadQueryFont: failed loading font '%s'\n",font_name);
      exit(-1);
  }

  /* assign the given font to our GC. */
  XSetFont(display, gc, font_info->fid);
  XSetFont(display, gc_bo, font_info->fid);

  PutXPM (display, img_w, gc_bo, "image.xpm");
  WriteText (display, msg_w, font_info, argv[1], gc, gc_bo);

  sleep(4);

  /* close the connection to the X server. */
  XCloseDisplay(display);

  return 1;
}
